#!/bin/bash

function showUsage {
    cat <<EOF

Usage:

   Generate Certificate Signing Request using openssl

   Purpose: Generate Cerificate Signing Request and Private Key files

            The csr file can be submitted to a CA server to obtaine a CA certificate.


  generate_csr -c <csr_file> -k <key_file> [-h <hostnames>] [-f <config_file>] [-i <ipaddrs>] [-V] [-K]

       -c <csr_file>     Required. Full path to output certificate signing request file.

       -k <key_file>     Required. Full path to output private key file.

       -f <config_file>  Optional. Configuration file generated with alternate name data. Defaults to /tmp/config.ssl

       -V                Optional. If specified generated signing request and private key files will be validated with openssl

       -K                Optional. If specified generated config file will not be deleted
       -O                Optional. If specified the SSL config file will be generated but the signing and key files will not be.

     	                           The config file may be modified to include country specific information, etc. and the openssl command 
	                           may be rerun with the modified config file.

       -h <hostnames>    Optional. List of one or more comma-separated hostnames for the certificate.

                         The first value will be used as the CN. All values will be used as SANs. If not 
                         specified, auto-detected short hostname and FQDN will be used.

       -i <ipaddrs>      Optional. List of one or more comma-separated IP addresses for the certificate.
   
                         All values will be used as SANs. If not specified, auto-detected IP addresses
                         will be used. Specify "none" to omit IP addresses from the certificate.
EOF
}

csrfile=
keyfile=
hostnames=
ipaddrs=
configfile= 
keepconfig=
verifyconfig=
configonly=

while getopts c:k:f:VKOh:i: flag
do
    case "${flag}" in
        c) csrfile=${OPTARG};;
        k) keyfile=${OPTARG};;
        f) configfile=${OPTARG};;       
        V) verifyconfig=1;;
        K) keepconfig=1;;
        O) configonly=1;;
        h) hostnames=${OPTARG};;
        i) ipaddrs=${OPTARG};;
    esac
done

if [[ -z $csrfile ]]; then
    echo " "
    echo "Error: A signing request file must be specified specified."
    showUsage
    exit 1
fi

if [[ -z $keyfile ]]; then
    echo " "
    echo "Error: A key file must be specified"
    showUsage
    exit 1
fi

if [[ -z $configfile ]]; then
   configfile=/tmp/config.ssl
fi

echo "CSR File    : $csrfile"
echo "Key File    : $keyfile"
echo "Config File : $configfile"
echo " "
   

if [[ -n $hostnames ]]; then
    hostnames=$(echo $hostnames | sed 's/,/ /g') # Replace commas with spaces
    hostnames=$(echo $hostnames | xargs) # Strip leading/trailing whitespace
    echo "HOSTNAMES: $hostnames"
else
    fqdn=$(hostname --fqdn)
    short=$(hostname -s)
    hostnames="$fqdn"
    if [[ $short != $fqdn ]]; then
        hostnames="$hostnames $short"
    fi
    echo "HOSTNAMES (AUTO-DETECTED): $hostnames"
    if [[ -z $hostnames ]]; then
        echo "Error: Failed to detect hostnames"
        exit 1
    fi
fi

if [[ -n $ipaddrs ]]; then
    ipaddrs=$(echo $ipaddrs | sed 's/,/ /g') # Replace commas with spaces
    ipaddrs=$(echo $ipaddrs | xargs) # Strip leading/trailing whitespace
    if [[ $ipaddrs == "none" ]]; then
        ipaddrs=
    fi
    echo "IPADDRS: $ipaddrs"
else
    ipaddrs="$(hostname -I)"
    echo "IPADDRS (AUTO-DETECTED): $ipaddrs"
    if [[ -z $ipaddrs ]]; then
        echo "Error: Failed to detect ipaddrs"
        exit 1
    fi
fi

COMMONNAME=$(echo $hostnames | awk '{print $1}') # CN = First hostname

# Create the openssl CER config file to avoid prompts
# CONFIGFILE=/tmp/config.ssl
#
# Note that noprompt is necessary in order to include alternate name data
#
# To change the country specific information, run with the -O argument to generate the config file without generating the signing request and key files,
# edit the config file, and rerun the openssl command with the modified config file
cat > $configfile <<EOF
[req]
default_bit = 4096
prompt = no
req_extensions = v3_req
distinguished_name = req_distinguished_name

[req_distinguished_name]
countryName             = US
stateOrProvinceName     = NY
localityName            = Armonk
organizationName        = IBM Corporation
commonName              = ${COMMONNAME}

[v3_req]
subjectAltName = @alt_names

[alt_names]
EOF

i=0
for san in $(echo $hostnames); do
    ((i=i+1))
    echo "DNS.$i = $san" >> $configfile
done

if [[ -n $ipaddrs ]]; then
    i=0
    for san in $(echo $ipaddrs); do
        ((i=i+1))
        echo "IP.$i = $san" >> $configfile
    done
fi

if [[ -n $configonly ]];  then 
   echo " "  
   echo "Generated $configfile" 
   echo " "
   echo "Command to generate signing request and key files: "
   echo "    openssl req -nodes -newkey rsa:2048 -out $csrfile -keyout $keyfile -config $configfile"
   echo " "
 exit 0
fi


if test -f "$csrfile"; then
   rm -f $csrfile
fi   

if test -f "$keyfile"; then
   rm -f $keyfile
fi

# Generate the new certificate signing request and key 
echo " "
echo "Generating Signing Request File $csrfile and key file $keyfile with config file $configfile ..."
echo " "
openssl req -nodes -newkey rsa:2048 -out $csrfile -keyout $keyfile -config $configfile

# Dump Certificate Signing Data to the Screen
if [[ -n $verifyconfig ]]; then 
 echo " "
 echo "Verifying Certificate Signing Request File $csrfile :"  
 echo " "
 openssl req -noout -text -in $csrfile
 echo " "
 echo "Verifying private key file $keyfile"
 echo " "
 openssl rsa -in $keyfile -check
fi

# Clean up
if [[ -z $keepconfig ]];  then 
   echo " "  
   echo  "Removing $configfile" 
   rm -f $configfile
fi

exit 0
